function z = WhittakerSmooth(y,  w, lambda, order)
% Weighted smoothing with a finite difference penalty
% y:      signal to be smoothed
% lambda: smoothing parameter 
% w:      weights (use one for missing values)
% order:      order of differences in penalty (generally 2)
% Paul Eilers, 2002
      if nargin < 4
          order=2;
      end
m = length(y);
W = spdiags(w, 0, m, m);
D = diff(speye(m), order);
C = chol(W+lambda * D' * D);
z = C \ (C' \ (w .* y));
end