close all
clear;

%% simulated spectral data
% peak
s1=30000*normpdf([-150:2:150],0,50);
s2=50000*normpdf([-150:2:150],0,40);
s3=30000*normpdf([-150:2:150],0,10);
s4=10000*normpdf([-150:2:150],0,40);
s5=15000*normpdf([-150:2:150],0,15);
s6=40000*normpdf([-150:2:150],0,20);
x=[s1,s2,s3,s4,s5,s6];
peak=x;
N=length(x);
n = [500:1:(500+N-1)]; 
% noise
randn('seed',0);
for i=1:10
    sign =30;
    noise(i,:) = (randn(N,1) * sign)';
end
% background
for i=1:30
    back(i,:)=50*i*sin((1:length(x))/400);
end
% simulated data
for i=1:10
simulated_data(i,:)=peak+back(i+20,:)+noise(i,:);
end
% plot simulated data
figure; set(gcf,'Position',[60 60 1200 350])
subplot(1,3,1)
plot(n, peak); xlabel('Wavenumber/cm^{-1}','fontsize',15); ylabel('Raman Intensity(a.u.)','fontsize',15); title('simulated peaks','fontsize',14);
subplot(1,3,2)
plot(n, back(20:30,:)); xlabel('Wavenumber/cm^{-1}','fontsize',15); ylabel('Raman Intensity(a.u.)','fontsize',15); title('simulated backgrounds','fontsize',14);
subplot(1,3,3)
plot(n, simulated_data); xlabel('Wavenumber/cm^{-1}','fontsize',15); ylabel('Raman Intensity(a.u.)','fontsize',15); title('simulated data','fontsize',14);
screen2png('simulated data.png');


%% compare methods
%%%%%%%%%%%%     airpls    %%%%%%%%%%%%
original_x=simulated_data;
lambda=10e5;
for i=1:size(original_x)
        y=original_x(i,:);
        [xbc, xb]= airPLS(y, lambda, 2, 0.1); % use airPLS to process each spectrm
        corrected_data(i,:)=xbc;       
        MSE(i,:)=sum((back(i+20,:)-xb).^2)./N; % calculate the MSE between real and estimated spectrum
end
figure; set(gcf,'Position',[60 60 1200 400])
subplot(1,2,1)
plot(n, simulated_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('simulated data');
subplot(1,2,2)
plot(n, corrected_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('airpls corrected data');
screen2png('airpls corrected data.png');
MSE_ave_airpls=mean(MSE)

%%%%%%%%%%%%     AS airpls    %%%%%%%%%%%%
original_x=simulated_data;
lambda=10e5;
order=2;
wep=0.1;
w=AS_air(simulated_data,lambda,order,wep); % calculate the weight of average spectrum by airPLS
for i=1:size(original_x)
        y=original_x(i,:);
        z= WhittakerSmooth(y', w, lambda, order );  % apply the weight to each spectrum
        corrected_data(i,:)=y-z';   
        MSE(i,:)=sum((back(i+20,:)-z').^2)./N; 
end
figure; set(gcf,'Position',[60 60 1200 400])
subplot(1,2,1)
plot(n, simulated_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('simulated data');
subplot(1,2,2)
plot(n, corrected_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('AS airpls corrected data');
screen2png('AS airpls corrected data.png');
MSE_ave_AS_airpls=mean(MSE)

%%%%%%%%%%%%     CW airpls    %%%%%%%%%%%%
original_x=simulated_data;
lambda=10e5;
for i=1:size(original_x)
        y=original_x(i,:);
         w=CW_air(y, lambda, 2,0.1);  % calculate the weight of each spectrum by using airpls
         w1(i,:)=w';  % get the weight vectors      
end
w_ave=mean(w1); % calculate the average of weight vector
order=2;

for i=1:size(original_x)
        y=original_x(i,:);
        z= WhittakerSmooth(y', w_ave', lambda, order ); % apply the average weight to each spectrum
        corrected_data(i,:)=y-z';   
        MSE(i,:)=sum((back(i+20,:)-z').^2)./N; 
end
figure; set(gcf,'Position',[60 60 1200 400])
subplot(1,2,1)
plot(n, simulated_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('simulated data');
subplot(1,2,2)
plot(n, corrected_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('CW airpls corrected data');
screen2png('CW airpls corrected data.png');
MSE_ave_CW_airpls=mean(MSE)

%%%%%%%%%%%%     mpls    %%%%%%%%%%%% 
original_x=simulated_data;
for i=1:size(original_x)
        y=original_x(i,:);
        z= MPLS(y,10e5,40,2); % use MPLS to process each spectrm
        corrected_data(i,:)=y-z';  
        MSE(i,:)=sum((back(i+20,:)-z').^2)./N; % calculate the MSE between real and estimated spectrum
end
figure; set(gcf,'Position',[60 60 1200 400])
subplot(1,2,1)
plot(n, simulated_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('simulated data');
subplot(1,2,2)
plot(n, corrected_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('mpls corrected data');
screen2png('mpls corrected data.png');
MSE_ave_mpls=mean(MSE)


%%%%%%%%%%%%     AS mpls    %%%%%%%%%%%%
original_x=simulated_data;
w=AS_MPLS(simulated_data, 40); % calculate the weight of average spectrum by MPLS
for i=1:size(original_x)
        y=original_x(i,:);
        z= WhittakerSmooth(y', w,10e5,2); % apply the weight to each spectrum
        corrected_data(i,:)=y-z';   
        MSE(i,:)=sum((back(i+20,:)-z').^2)./N; 
end
figure; set(gcf,'Position',[60 60 1200 400])
subplot(1,2,1)
plot(n, simulated_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('simulated data');
subplot(1,2,2)
plot(n, corrected_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('AS mpls corrected data');
screen2png('AS mpls corrected data.png');
MSE_ave_AS_mpls=mean(MSE)


%%%%%%%%%%%%     CW mpls    %%%%%%%%%%%%
original_x=simulated_data;
ave_w=CW_MPLS(simulated_data,40);  %calculate the weight of each spectrum by using mpls and  the ave_w is obtained from the average of weight vector
for i=1:size(original_x)
        y=original_x(i,:);
        z= WhittakerSmooth(y', w,10e5,2);  % apply the weight to each spectrum
        corrected_data(i,:)=y-z';  
        MSE(i,:)=sum((back(i+20,:)-z').^2)./N; 
end
figure; set(gcf,'Position',[60 60 1200 400])
subplot(1,2,1)
plot(n, simulated_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('simulated data');
subplot(1,2,2)
plot(n, corrected_data) ; xlabel('Wavenumber/cm^{-1}'); ylabel('Raman Intensity(a.u.)');
title('CW mpls corrected data');
screen2png('CW mpls corrected data.png');
MSE_ave_CW_mpls=mean(MSE)

%% output MSE results
result_MSE=[
    MSE_ave_airpls;
    MSE_ave_AS_airpls;
    MSE_ave_CW_airpls    
    MSE_ave_mpls;
    MSE_ave_AS_mpls;
    MSE_ave_CW_mpls;]

